@@ -119,8 +119,6 @@ void           gnome_vfs_async_set_file_info          (GnomeVFSAsyncHandle
 void           gnome_vfs_async_load_directory         (GnomeVFSAsyncHandle                  **handle_return,
 						       const gchar                           *text_uri,
 						       GnomeVFSFileInfoOptions                options,
-						       GnomeVFSDirectorySortRule              sort_rules[],
-						       gboolean                               reverse_order,
 						       GnomeVFSDirectoryFilterType            filter_type,
 						       GnomeVFSDirectoryFilterOptions         filter_options,
 						       const gchar                           *filter_pattern,
@@ -130,8 +128,6 @@ void           gnome_vfs_async_load_directory         (GnomeVFSAsyncHandle
 void           gnome_vfs_async_load_directory_uri     (GnomeVFSAsyncHandle                  **handle_return,
 						       GnomeVFSURI                           *uri,
 						       GnomeVFSFileInfoOptions                options,
-						       GnomeVFSDirectorySortRule              sort_rules[],
-						       gboolean                               reverse_order,
 						       GnomeVFSDirectoryFilterType            filter_type,
 						       GnomeVFSDirectoryFilterOptions         filter_options,
 						       const gchar                           *filter_pattern,
