@@ -30,8 +30,10 @@
 #include "gnome-vfs-mime.h"
 #include "gnome-vfs-mime-info.h"
 #include "gnome-vfs-mime-sniff-buffer.h"
+#include "gnome-vfs-mime-sniff-buffer-private.h"
 #include "gnome-vfs-mime-private.h"
 #include "gnome-vfs-module-shared.h"
+#include "gnome-vfs-private-utils.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -49,6 +51,8 @@ static gboolean module_inited = FALSE;
 static GHashTable *mime_extensions [2] = { NULL, NULL };
 static GList      *mime_regexs     [2] = { NULL, NULL };
 
+#define DEFAULT_DATE_TRACKER_INTERVAL	5	/* in milliseconds */
+
 typedef struct {
 	char *mime_type;
 	regex_t regex;
@@ -56,18 +60,28 @@ typedef struct {
 
 typedef struct {
 	char *dirname;
-	struct stat s;
 	unsigned int valid : 1;
 	unsigned int system_dir : 1;
 } mime_dir_source_t;
 
+typedef struct {
+	char *file_path;
+	time_t mtime;
+} FileDateRecord;
+
+struct FileDateTracker {
+	time_t last_checked;
+	guint check_interval;
+	GHashTable *records;
+};
+
 /* chached localhostname */
 static char localhostname[1024];
 static gboolean got_localhostname = FALSE;
 
 /* These ones are used to automatically reload mime-types on demand */
 static mime_dir_source_t gnome_mime_dir, user_mime_dir;
-static time_t last_checked;
+static FileDateTracker *mime_data_date_tracker;
 
 #ifdef G_THREADS_ENABLED
 
@@ -76,19 +90,18 @@ G_LOCK_DEFINE_STATIC (mime_mutex);
 
 #endif /* G_LOCK_DEFINE_STATIC */
 
-
 
 static char *
 get_priority (char *def, int *priority)
 {
 	*priority = 0;
 
-	if (*def == ','){
+	if (*def == ',') {
 		def++;
-		if (*def == '1'){
+		if (*def == '1') {
 			*priority = 0;
 			def++;
-		} else if (*def == '2'){
+		} else if (*def == '2') {
 			*priority = 1;
 			def++;
 		}
@@ -100,10 +113,10 @@ get_priority (char *def, int *priority)
 	return def;
 }
 
-static gint
+static int
 list_find_type (gconstpointer value, gconstpointer type)
 {
-	return (g_strcasecmp( (const gchar*) value, (const gchar*) type ));
+	return g_strcasecmp((const char *) value, (const char *) type);
 }
 
 static void
@@ -120,9 +133,9 @@ add_to_key (char *mime_type, char *def)
 		def = get_priority (def, &priority);
 		s = p = g_strdup (def);
 
-		while ((ext = strtok_r (s, " \t\n\r,", &tokp)) != NULL){
-			list = (GList*) g_hash_table_lookup (mime_extensions [priority], ext);
-		    if (!g_list_find_custom (list, (gpointer) mime_type, list_find_type)) {
+		while ((ext = strtok_r (s, " \t\n\r,", &tokp)) != NULL) {
+			list = (GList *) g_hash_table_lookup (mime_extensions [priority], ext);
+			if (!g_list_find_custom (list, mime_type, list_find_type)) {
 				list = g_list_prepend (list, g_strdup (mime_type));
 				g_hash_table_insert (mime_extensions [priority], g_strdup (ext), list);
 			}
@@ -131,23 +144,25 @@ add_to_key (char *mime_type, char *def)
 		g_free (p);
 	}
 
-	if (strncmp (def, "regex", 5) == 0){
+	if (strncmp (def, "regex", 5) == 0) {
 		RegexMimePair *mp;
 		def += 5;
 		def = get_priority (def, &priority);
 
-		while (*def && isspace ((unsigned char)*def))
+		while (*def != '\0' && isspace ((unsigned char)*def)) {
 			def++;
+		}
 
-		if (!*def)
+		if (*def == '\0') {
 			return;
+		}
 
 		/* This was g_new instead of g_new0, but there seems
 		 * to be a bug in the Solaris? version of regcomp that
 		 * requires an initialized regex or it will crash.
 		 */
 		mp = g_new0 (RegexMimePair, 1);
-		if (regcomp (&mp->regex, def, REG_EXTENDED | REG_NOSUB)){
+		if (regcomp (&mp->regex, def, REG_EXTENDED | REG_NOSUB)) {
 			g_free (mp);
 			return;
 		}
@@ -160,34 +175,37 @@ add_to_key (char *mime_type, char *def)
 static void
 mime_fill_from_file (const char *filename)
 {
-	FILE *f;
+	FILE *file;
 	char buf [1024];
 	char *current_key;
 
 	g_assert (filename != NULL);
 
-	f = fopen (filename, "r");
+	file = fopen (filename, "r");
 
-	if (!f)
+	if (file == NULL) {
 		return;
-
+	}
+	
 	current_key = NULL;
-	while (fgets (buf, sizeof (buf), f)){
+	while (fgets (buf, sizeof (buf), file) != NULL) {
 		char *p;
 
-		if (buf [0] == '#')
+		if (buf [0] == '#') {
 			continue;
-
-		/* Trim trailing spaces */
-		for (p = buf + strlen (buf) - 1; p >= buf; p--){
-			if (isspace ((unsigned char)*p) || *p == '\n')
-				*p = 0;
-			else
-				break;
 		}
 
-		if (!buf [0])
+		/* Trim trailing spaces */
+		for (p = buf + strlen (buf) - 1; p >= buf; p--) {
+			if (!isspace ((unsigned char)*p) && *p != '\n') {
+				break;
+			}
+			*p = 0;
+		}
+
+		if (buf [0] == '\0') {
 			continue;
+		}
 
 		if (buf [0] == '\t' || buf [0] == ' '){
 			if (current_key){
@@ -212,7 +230,9 @@ mime_fill_from_file (const char *filename)
 
 	g_free (current_key);
 
-	fclose (f);
+	fclose (file);
+
+	gnome_vfs_file_date_tracker_start_tracking_file (mime_data_date_tracker, filename);
 }
 
 static void
@@ -222,48 +242,57 @@ mime_load (mime_dir_source_t *source)
 	struct dirent *dent;
 	const int extlen = sizeof (".mime") - 1;
 	char *filename;
+	struct stat s;
 
 	g_return_if_fail (source != NULL);
 	g_return_if_fail (source->dirname != NULL);
 
-	if (stat (source->dirname, &source->s) != -1)
-		source->valid = TRUE;
-	else
-		source->valid = FALSE;
+	source->valid = (stat (source->dirname, &s) != -1);
 
 	dir = opendir (source->dirname);
-	if (!dir){
+	if (dir == NULL) {
 		source->valid = FALSE;
 		return;
 	}
 
-	if (source->system_dir){
+	if (source->system_dir) {
 		filename = g_strconcat (source->dirname, "/gnome-vfs.mime", NULL);
 		mime_fill_from_file (filename);
 		g_free (filename);
 	}
 
-	while ((dent = readdir (dir)) != NULL){
+	while (TRUE) {
+		int len;
+		
+		dent = readdir (dir);
+		if (dent == NULL) {
+			break;
+		}
+		
+		len = strlen (dent->d_name);
 
-		int len = strlen (dent->d_name);
-
-		if (len <= extlen)
+		if (len <= extlen) {
 			continue;
-		if (strcmp (dent->d_name + len - extlen, ".mime"))
+		}
+		
+		if (strcmp (dent->d_name + len - extlen, ".mime") != 0) {
 			continue;
+		}
 
-		if (source->system_dir && !strcmp (dent->d_name, "gnome-vfs.mime"))
+		if (source->system_dir && strcmp (dent->d_name, "gnome-vfs.mime") == 0) {
 			continue;
+		}
 
-		if (source->system_dir && !strcmp (dent->d_name, "gnome.mime")) {
+		if (source->system_dir && strcmp (dent->d_name, "gnome.mime") == 0) {
 			/* Ignore the obsolete "official" one so it doesn't override
 			 * the new official one.
 			 */
 			continue;
 		}
 
-		if (!source->system_dir && !strcmp (dent->d_name, "user.mime"))
+		if (!source->system_dir && strcmp (dent->d_name, "user.mime") == 0) {
 			continue;
+		}
 
 		filename = g_strconcat (source->dirname, "/", dent->d_name, NULL);
 
@@ -277,6 +306,8 @@ mime_load (mime_dir_source_t *source)
 		mime_fill_from_file (filename);
 		g_free (filename);
 	}
+
+	gnome_vfs_file_date_tracker_start_tracking_file (mime_data_date_tracker, source->dirname);
 }
 
 static gboolean
@@ -315,31 +346,14 @@ mime_extensions_empty (void)
 static void
 maybe_reload (void)
 {
-	time_t now = time (NULL);
-	gboolean need_reload = FALSE;
-	struct stat s;
-
-	if (last_checked + 5 >= now)
-		return;
-
-	if (stat (gnome_mime_dir.dirname, &s) != -1)
-		if (s.st_mtime != gnome_mime_dir.s.st_mtime)
-			need_reload = TRUE;
-
-	if (stat (user_mime_dir.dirname, &s) != -1)
-		if (s.st_mtime != user_mime_dir.s.st_mtime)
-			need_reload = TRUE;
-
-	last_checked = now;
-
-	if (!need_reload)
+	if (!gnome_vfs_file_date_tracker_date_has_changed (mime_data_date_tracker)) {
 		return;
+	}
 
 	mime_extensions_empty ();
 
 	mime_load (&gnome_mime_dir);
 	mime_load (&user_mime_dir);
-	last_checked = time (NULL);
 }
 
 static void
@@ -347,6 +361,8 @@ mime_init (void)
 {
 	mime_extensions [0] = g_hash_table_new (g_str_hash, g_str_equal);
 	mime_extensions [1] = g_hash_table_new (g_str_hash, g_str_equal);
+
+	mime_data_date_tracker = gnome_vfs_file_date_tracker_new ();
 	
 	gnome_mime_dir.dirname = g_strconcat (GNOME_VFS_DATADIR, "/mime-info", NULL);
 	gnome_mime_dir.system_dir = TRUE;
@@ -356,7 +372,6 @@ mime_init (void)
 
 	mime_load (&gnome_mime_dir);
 	mime_load (&user_mime_dir);
-	last_checked = time (NULL);
 
 	module_inited = TRUE;
 }
@@ -374,6 +389,8 @@ gnome_vfs_mime_shutdown (void)
 	
 	g_hash_table_destroy (mime_extensions[0]);
 	g_hash_table_destroy (mime_extensions[1]);
+
+	gnome_vfs_file_date_tracker_free (mime_data_date_tracker);
 	
 	g_free (gnome_mime_dir.dirname);
 	g_free (user_mime_dir.dirname);
@@ -391,23 +408,27 @@ gnome_vfs_mime_shutdown (void)
  * determined, it will return @defaultv.
  */
 const char *
-gnome_vfs_mime_type_from_name_or_default (const gchar *filename, const gchar *defaultv)
+gnome_vfs_mime_type_from_name_or_default (const char *filename, const char *defaultv)
 {
 	const gchar *ext;
-	gchar *upext;
+	char *upext;
 	int priority;
-	const gchar *result = defaultv;
+	const char *result = defaultv;
+
+	if (filename == NULL) {
+		return result;
+	}
 
 	G_LOCK (mime_mutex);
 
-	if (!filename)
-		goto done;
 	ext = strrchr (filename, '.');
-	if (ext)
+	if (ext != NULL) {
 		++ext;
-
-	if (!module_inited)
+	}
+	
+	if (!module_inited) {
 		mime_init ();
+	}
 
 	maybe_reload ();
 
@@ -415,34 +436,34 @@ gnome_vfs_mime_type_from_name_or_default (const gchar *filename, const gchar *de
 		GList *l;
 		GList *list = NULL ;
 		
-		if (ext){
+		if (ext != NULL) {
 			
 			list = g_hash_table_lookup (mime_extensions [priority], ext);
-			if (list) {
+			if (list != NULL) {
 				list = g_list_first( list );
-				result = (gchar *) list->data;
-				goto done;
+				result = (const char *) list->data;
+				break;
 			}
 
 			/* Search for UPPER case extension */
 			upext = g_strdup (ext);
 			g_strup (upext);
 			list = g_hash_table_lookup (mime_extensions [priority], upext);
-			if (list) {
+			if (list != NULL) {
 				g_free (upext);
 				list = g_list_first (list);
-				result = (gchar *) list->data;
-				goto done;
+				result = (const char *) list->data;
+				break;
 			}
 
 			/* Final check for lower case */
 			g_strdown (upext);
 			list = g_hash_table_lookup (mime_extensions [priority], upext);
  			g_free (upext);
-			if (list) {
+			if (list != NULL) {
 				list = g_list_first (list);
-				result = (gchar *) list->data;
-				goto done;
+				result = (const char *) list->data;
+				break;
 			}
 		}
 
@@ -451,12 +472,12 @@ gnome_vfs_mime_type_from_name_or_default (const gchar *filename, const gchar *de
 
 			if (regexec (&mp->regex, filename, 0, 0, 0) == 0) {
 				result = mp->mime_type;
-				goto done;
+				G_UNLOCK (mime_mutex);
+				return result;
 			}
 		}
 	}
 
- done:
 	G_UNLOCK (mime_mutex);
 	return result;
 }
@@ -482,12 +503,74 @@ gnome_vfs_get_mime_type_from_uri_internal (GnomeVFSURI *uri)
 
 	/* Return a mime type based on the file extension or NULL if no match. */
 	base_name = gnome_vfs_uri_get_basename (uri);
-	if (base_name == NULL)
+	if (base_name == NULL) {
 		return NULL;
+	}
 
 	return gnome_vfs_mime_type_from_name_or_default (base_name, NULL);
 }
 
+const char *
+gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *file_name)
+{
+	const char *result;
+
+	result = NULL;
+	
+	if (buffer != NULL) {
+		result = gnome_vfs_mime_get_type_from_magic_table (buffer);
+		
+		if (result != NULL) {
+			return result;
+		}
+		
+		/* Special handling of gzip files -- use the file name to make a more
+		 * accurate guess of the file type for formats such as gnumeric.gz and
+		 * pdf.gz. Without this, these would always get identified as gzip even though
+		 * their name would suggest otherwise.
+		 * FIXME bugzilla.eazel.com 6867:
+		 * Generalize this so that we can have different magic patters
+		 * other than gzip do this.
+		 */
+		if (gnome_vfs_sniff_buffer_looks_like_gzip (buffer, file_name)) {
+			return "application/x-gzip";
+		}
+		
+		if (result == NULL) {
+			if (gnome_vfs_sniff_buffer_looks_like_text (buffer)) {
+				/* Text file -- treat extensions as a more accurate source
+				 * of type information.
+				 */
+				
+				if (file_name != NULL) {
+					result = gnome_vfs_mime_type_from_name_or_default (file_name, NULL);
+				}
+	
+				if (result != NULL) {
+					return result;
+				}
+
+				/* Didn't find an extension match, assume plain text. */
+				return "text/plain";
+
+			} else if (gnome_vfs_sniff_buffer_looks_like_mp3 (buffer)) {
+				return "audio/x-mp3";
+			}
+		}
+	}
+	
+	if (result == NULL && file_name != NULL) {
+		/* No type recognized -- fall back on extensions. */
+		result = gnome_vfs_mime_type_from_name_or_default (file_name, NULL);
+	}
+	
+	if (result == NULL) {
+		result = GNOME_VFS_MIME_TYPE_UNKNOWN;
+	}
+	
+	return result;
+}
+
 /**
  * gnome_vfs_get_mime_type:
  * @uri: a real file or a non-existent uri.
@@ -523,30 +606,8 @@ gnome_vfs_get_mime_type (GnomeVFSURI *uri)
 	
 	buffer = gnome_vfs_mime_sniff_buffer_new_from_handle (handle);
 
-	/* check the type from the file data */
-	result = gnome_vfs_get_mime_type_for_buffer (buffer);
+	result = gnome_vfs_get_mime_type_internal (buffer, gnome_vfs_uri_get_basename (uri));
 
-	if (result == NULL) {
-		if (gnome_vfs_sniff_buffer_looks_like_text (buffer)) {
-			/* Text file -- treat extensions as a more accurate source
-			 * of type information.
-			 */
-			result = gnome_vfs_get_mime_type_from_uri_internal (uri);
-			if (result == NULL) {
-				/* Didn't find an extension match, assume plain text. */
-				result = "text/plain";
-			}
-		} else {
-			/* No type recognized -- fall back on extensions. */
-			result = gnome_vfs_get_mime_type_from_uri_internal (uri);
-		}
-	}
-
-	if (result == NULL) {
-		/* no type detected, return a generic file type */
-		result = GNOME_VFS_MIME_TYPE_UNKNOWN;
-	}
-	
 	gnome_vfs_mime_sniff_buffer_free (buffer);
 	gnome_vfs_close (handle);
 
@@ -603,7 +664,7 @@ gnome_vfs_get_file_mime_type (const char *path, const struct stat *stat_info,
 	struct stat tmp_stat_buffer;
 	FILE *file;
 
-	buffer = NULL;
+	file = NULL;
 	result = NULL;
 
 	/* get the stat info if needed */
@@ -613,52 +674,39 @@ gnome_vfs_get_file_mime_type (const char *path, const struct stat *stat_info,
 
 	/* single out special file types */
 	if (stat_info && !S_ISREG(stat_info->st_mode)) {
-		if (S_ISDIR(stat_info->st_mode))
+		if (S_ISDIR(stat_info->st_mode)) {
 			return "x-directory/normal";
-		else if (S_ISCHR(stat_info->st_mode))
+		} else if (S_ISCHR(stat_info->st_mode)) {
 			return "x-special/device-char";
-		else if (S_ISBLK(stat_info->st_mode))
+		} else if (S_ISBLK(stat_info->st_mode)) {
 			return "x-special/device-block";
-		else if (S_ISFIFO(stat_info->st_mode))
+		} else if (S_ISFIFO(stat_info->st_mode)) {
 			return "x-special/fifo";
-		else if (S_ISSOCK(stat_info->st_mode))
+		} else if (S_ISSOCK(stat_info->st_mode)) {
 			return "x-special/socket";
-		else
+		} else {
 			/* unknown entry type, return generic file type */
 			return GNOME_VFS_MIME_TYPE_UNKNOWN;
+		}
 	}
 
 	if (!suffix_only) {
 		file = fopen(path, "r");
-		if (file != NULL) {
-			buffer = gnome_vfs_mime_sniff_buffer_new_generic
-				(file_seek_binder, file_read_binder, file);
-
-			result = gnome_vfs_get_mime_type_for_buffer (buffer);
-			if (result == NULL && gnome_vfs_sniff_buffer_looks_like_text (buffer)) {
-				/* Text file -- treat extensions as a more accurate source
-				 * of type information.
-				 */
-				result = gnome_vfs_mime_type_from_name_or_default (path, NULL);
-				if (result == NULL) {
-					/* Didn't find an extension match, assume plain text. */
-					result = "text/plain";
-				}
-			}
-			fclose (file);
-		}
 	}
 
-	if (result == NULL) {
-		result = gnome_vfs_mime_type_from_name_or_default (path, NULL);	
-	}
-	if (result == NULL) {
-		result = GNOME_VFS_MIME_TYPE_UNKNOWN;
-	}
-	if (buffer) {
+	if (file != NULL) {
+		buffer = gnome_vfs_mime_sniff_buffer_new_generic
+			(file_seek_binder, file_read_binder, file);
+
+		result = gnome_vfs_get_mime_type_internal (buffer, path);
 		gnome_vfs_mime_sniff_buffer_free (buffer);
+		fclose (file);
+	} else {
+		result = gnome_vfs_get_mime_type_internal (NULL, path);
 	}
 
+	
+	g_assert (result != NULL);
 	return result;
 }
 
@@ -706,23 +754,13 @@ gnome_vfs_get_mime_type_from_file_data (GnomeVFSURI *uri)
 	error = gnome_vfs_open_uri (&handle, uri, GNOME_VFS_OPEN_READ);
 
 	if (error != GNOME_VFS_OK) {
-		return NULL;
+		return GNOME_VFS_MIME_TYPE_UNKNOWN;
 	}
 	
 	buffer = gnome_vfs_mime_sniff_buffer_new_from_handle (handle);
 
-	/* check the type from the file data */
-	result = gnome_vfs_get_mime_type_for_buffer (buffer);
+	result = gnome_vfs_get_mime_type_internal (buffer, NULL);	
 
-	if (result == NULL && gnome_vfs_sniff_buffer_looks_like_text (buffer)) {
-		result = "text/plain";
-	}
-
-	if (result == NULL) {
-		/* no type detected, return a generic file type */
-		result = GNOME_VFS_MIME_TYPE_UNKNOWN;
-	}
-	
 	gnome_vfs_mime_sniff_buffer_free (buffer);
 	gnome_vfs_close (handle);
 
@@ -748,12 +786,63 @@ gnome_vfs_get_mime_type_for_data (gconstpointer data, int data_size)
 	buffer = gnome_vfs_mime_sniff_buffer_new_from_existing_data
 		(data, data_size);
 
-	result = gnome_vfs_get_mime_type_for_buffer (buffer);
+	result = gnome_vfs_get_mime_type_internal (buffer, NULL);	
+
 	gnome_vfs_mime_sniff_buffer_free (buffer);
 
 	return result;
 }
 
+gboolean
+gnome_vfs_mime_type_is_supertype (const char *mime_type)
+{
+	int length;
+
+	if (mime_type == NULL) {
+		return FALSE;
+	}
+
+	length = strlen (mime_type);
+
+	return length > 2
+	       && mime_type[length - 2] == '/' 
+	       && mime_type[length - 1] == '*';
+}
+
+static char *
+extract_prefix_add_suffix (const char *string, const char *separator, const char *suffix)
+{
+        const char *separator_position;
+        int prefix_length;
+        char *result;
+
+        separator_position = strstr (string, separator);
+        prefix_length = separator_position == NULL
+                ? strlen (string)
+                : separator_position - string;
+
+        result = g_malloc (prefix_length + strlen (suffix) + 1);
+        
+        strncpy (result, string, prefix_length);
+        result[prefix_length] = '\0';
+
+        strcat (result, suffix);
+
+        return result;
+}
+
+/* Returns the supertype for a mime type. Note that if called
+ * on a supertype it will return a copy of the supertype.
+ */
+char *
+gnome_vfs_get_supertype_from_mime_type (const char *mime_type)
+{
+	if (mime_type == NULL) {
+		return NULL;
+	}
+        return extract_prefix_add_suffix (mime_type, "/", "/*");
+}
+
 /**
  * gnome_uri_list_extract_uris:
  * @uri_list: an uri-list in the standard format.
@@ -765,7 +854,7 @@ gnome_vfs_get_mime_type_for_data (gconstpointer data, int data_size)
  * returned list
  */
 GList*
-gnome_uri_list_extract_uris (const gchar* uri_list)
+gnome_uri_list_extract_uris (const char *uri_list)
 {
 	const guchar *p, *q;
 	gchar *retval;
@@ -773,26 +862,29 @@ gnome_uri_list_extract_uris (const gchar* uri_list)
 
 	g_return_val_if_fail (uri_list != NULL, NULL);
 
-	p = (const guchar *)uri_list;
+	p = (const guchar *) uri_list;
 
 	/* We don't actually try to validate the URI according to RFC
 	 * 2396, or even check for allowed characters - we just ignore
 	 * comments and trim whitespace off the ends.  We also
 	 * allow LF delimination as well as the specified CRLF.
 	 */
-	while (p) {
+	while (p != NULL) {
 		if (*p != '#') {
-			while (isspace(*p))
+			while (isspace (*p)) {
 				p++;
+			}
 
 			q = p;
-			while (*q && (*q != '\n') && (*q != '\r'))
+			while (*q != '\0' && *q != '\n' && *q != '\r') {
 				q++;
+			}
 
 			if (q > p) {
 			        q--;
-				while (q > p && isspace(*q))
+				while (q > p && isspace (*q)) {
 					q--;
+				}
 
 				retval = g_malloc (q - p + 2);
 				strncpy (retval, p, q - p + 1);
@@ -802,8 +894,9 @@ gnome_uri_list_extract_uris (const gchar* uri_list)
 			}
 		}
 		p = strchr (p, '\n');
-		if (p)
+		if (p != NULL) {
 			p++;
+		}
 	}
 
 	return g_list_reverse (result);
@@ -822,7 +915,7 @@ gnome_uri_list_extract_uris (const gchar* uri_list)
  * returned list.
  */
 GList*
-gnome_uri_list_extract_filenames (const gchar* uri_list)
+gnome_uri_list_extract_filenames (const char *uri_list)
 {
 	GList *tmp_list, *node, *result;
 
@@ -831,8 +924,8 @@ gnome_uri_list_extract_filenames (const gchar* uri_list)
 	result = gnome_uri_list_extract_uris (uri_list);
 
 	tmp_list = result;
-	while (tmp_list) {
-		gchar *s = tmp_list->data;
+	while (tmp_list != NULL) {
+		char *s = tmp_list->data;
 
 		node = tmp_list;
 		tmp_list = tmp_list->next;
@@ -859,57 +952,70 @@ gnome_uri_list_extract_filenames (const gchar* uri_list)
  * Returns: a newly allocated string if the @uri, or %NULL
  * if the @uri was not a local file
  */
-gchar *
-gnome_uri_extract_filename (const gchar* uri)
+char *
+gnome_uri_extract_filename (const char *uri)
 {
+	char *hostname;
+	char *p, *result;
+	const char *path;
+
 	/* file uri with a hostname */
-	if (strncmp(uri, "file://", strlen("file://"))==0) {
-		char *hostname = g_strdup(&uri[strlen("file://")]);
-		char *p = strchr(hostname,'/');
-		char *path;
-		/* if we can't find the '/' this uri is bad */
-		if(!p) {
-			g_free(hostname);
+	if (gnome_vfs_istr_has_prefix (uri, "file://")) {
+
+		hostname = g_strdup (&uri[strlen ("file://")]);
+		/* if we can't find the '/' this uri is bad */		
+		p = strchr (hostname,'/');
+		if (p == NULL) {
+			g_free (hostname);
 			return NULL;
 		}
-		/* if no hostname */
-		if(p==hostname)
-			return hostname;
 
-		path = g_strdup(p);
+		/* if no hostname */
+		if (p == hostname) {
+			return hostname;
+		}
+
+		result = g_strdup (p);
 		*p = '\0';
 
 		/* gel local host name and cache it */
-		if(!got_localhostname) {
+		if (!got_localhostname) {
 			G_LOCK (mime_mutex);
-			if(gethostname(localhostname,
-				       sizeof(localhostname)) < 0) {
-				strcpy(localhostname,"");
+			if (gethostname (localhostname, sizeof (localhostname)) < 0) {
+				strcpy (localhostname, "");
 			}
 			got_localhostname = TRUE;
 			G_UNLOCK (mime_mutex);
 		}
 
 		/* if really local */
-		if((localhostname[0] &&
-		    g_strcasecmp(hostname,localhostname)==0) ||
-		   g_strcasecmp(hostname,"localhost")==0) {
-			g_free(hostname);
-			return path;
+		if (g_strcasecmp (hostname, localhostname) == 0
+			|| g_strcasecmp (hostname, "localhost") == 0) {
+			g_free (hostname);
+			return result;
 		}
 		
-		g_free(hostname);
-		g_free(path);
+		g_free (hostname);
+		g_free (result);
 		return NULL;
 
-	/* if the file doesn't have the //, we take it containing 
-	   a local path */
-	} else if (strncmp(uri, "file:", strlen("file:"))==0) {
-		const char *path = &uri[strlen("file:")];
-		/* if empty bad */
-		if(!*path) return NULL;
-		return g_strdup(path);
 	}
+	
+	/* if the file doesn't have the //, we take it containing 
+	 * a local path
+	 */
+	if (gnome_vfs_istr_has_prefix (uri, "file:")) {
+		
+		path = &uri[strlen ("file:")];
+
+		/* if empty bad */
+		if (*path == '\0') {
+			return NULL;
+		}
+
+		return g_strdup (path);
+	}
+
 	return NULL;
 }
 
@@ -925,3 +1031,128 @@ gnome_uri_list_free_strings (GList *list)
 	g_list_foreach (list, (GFunc) g_free, NULL);
 	g_list_free (list);
 }
+
+static void
+file_date_record_update_mtime (FileDateRecord *record)
+{
+	struct stat s;
+
+	if (stat (record->file_path, &s) != -1) {
+		record->mtime = s.st_mtime;
+	}	
+}
+
+static FileDateRecord *
+file_date_record_new (const char *file_path) {
+	FileDateRecord *record;
+
+	record = g_new0 (FileDateRecord, 1);
+	record->file_path = g_strdup (file_path);
+
+	file_date_record_update_mtime (record);
+
+	return record;
+}
+
+static void
+file_date_record_free (FileDateRecord *record)
+{
+	g_free (record->file_path);
+	g_free (record);
+}
+
+FileDateTracker *
+gnome_vfs_file_date_tracker_new (void)
+{
+	FileDateTracker *tracker;
+
+	tracker = g_new0 (FileDateTracker, 1);
+	tracker->check_interval = DEFAULT_DATE_TRACKER_INTERVAL;
+	tracker->records = g_hash_table_new (g_str_hash, g_str_equal);
+
+	return tracker;
+}
+
+static gboolean
+release_key_and_value (gpointer key, gpointer value, gpointer user_data)
+{
+	g_free (key);
+	file_date_record_free (value);
+
+	return TRUE;
+}
+
+void
+gnome_vfs_file_date_tracker_free (FileDateTracker *tracker)
+{
+	g_hash_table_foreach_remove (tracker->records, release_key_and_value, NULL);
+	g_hash_table_destroy (tracker->records);
+	g_free (tracker);
+}
+
+/*
+ * Record the current mod date for a specified file, so that we can check
+ * later whether it has changed.
+ */
+void
+gnome_vfs_file_date_tracker_start_tracking_file (FileDateTracker *tracker, 
+				                 const char *local_file_path)
+{
+	FileDateRecord *record;
+
+	record = g_hash_table_lookup (tracker->records, local_file_path);
+	if (record != NULL) {
+		file_date_record_update_mtime (record);
+	} else {
+		g_hash_table_insert (tracker->records, 
+				     g_strdup (local_file_path), 
+				     file_date_record_new (local_file_path));
+	}
+}
+
+static void 
+check_and_update_one (gpointer key, gpointer value, gpointer user_data)
+{
+	FileDateRecord *record;
+	gboolean *return_has_changed;
+	struct stat s;
+
+	g_assert (key != NULL);
+	g_assert (value != NULL);
+	g_assert (user_data != NULL);
+
+	record = (FileDateRecord *)value;
+	return_has_changed = (gboolean *)user_data;
+
+	if (stat (record->file_path, &s) != -1) {
+		if (s.st_mtime != record->mtime) {
+			record->mtime = s.st_mtime;
+			*return_has_changed = TRUE;
+		}
+	}
+}
+
+gboolean
+gnome_vfs_file_date_tracker_date_has_changed (FileDateTracker *tracker)
+{
+	time_t now;
+	gboolean any_date_changed;
+
+	now = time (NULL);
+
+	/* Note that this might overflow once in a blue moon, but the
+	 * only side-effect of that would be a slightly-early check
+	 * for changes.
+	 */
+	if (tracker->last_checked + tracker->check_interval >= now) {
+		return FALSE;
+	}
+
+	any_date_changed = FALSE;
+
+	g_hash_table_foreach (tracker->records, check_and_update_one, &any_date_changed);
+
+	tracker->last_checked = now;
+
+	return any_date_changed;
+}
